import unittest
from dataclasses import dataclass, field
from typing import List, Optional

import marshmallow.validate

import marshmallow_dataclass


@dataclass
class Building:
    # field metadata is used to instantiate the marshmallow field
    height: float = field(metadata={"validate": marshmallow.validate.Range(min=0)})
    name: str = field(default="anonymous")


@dataclass
class City:
    name: Optional[str]
    buildings: List[Building] = field(default_factory=list)


CitySchema = marshmallow_dataclass.class_schema(City)()


class TestCityBuildingExample(unittest.TestCase):
    def test_load(self):
        city = CitySchema.load(
            {"name": "Paris", "buildings": [{"name": "Eiffel Tower", "height": 324}]}
        )
        self.assertEqual(
            city,
            City(name="Paris", buildings=[Building(height=324.0, name="Eiffel Tower")]),
        )

    def test_back_and_forth(self):
        original_dict = {
            "name": "Paris",
            "buildings": [{"name": "Eiffel Tower", "height": 324}],
        }
        city_dict = CitySchema.dump(CitySchema.load(original_dict))
        self.assertEqual(city_dict, original_dict)


if __name__ == "__main__":
    unittest.main()



from dataclasses import field
import sys
import unittest
from typing import List, Optional, Union, Dict

import marshmallow

from marshmallow_dataclass import dataclass


class TestClassSchema(unittest.TestCase):
    def test_simple_union(self):
        @dataclass
        class IntOrStr:
            value: Union[int, str]

        schema = IntOrStr.Schema()
        data_in = {"value": "hello"}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, IntOrStr(value="hello"))
        self.assertEqual(schema.dump(loaded), data_in)

        data_in = {"value": 42}
        self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_list_union_builtin(self):
        @dataclass
        class Dclass2:
            value: List[Union[int, str]]

        schema = Dclass2.Schema()
        data_in = {"value": ["hello", 42]}
        self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_list_union_object(self):
        @dataclass
        class Elm1:
            elm1: str

        @dataclass
        class Elm2:
            elm2: str

        @dataclass
        class Dclass:
            value: List[Union[Elm1, Elm2]]

        schema = Dclass.Schema()
        data_in = {"value": [{"elm1": "foo"}, {"elm2": "bar"}]}
        load = schema.load(data_in)
        self.assertEqual(load, Dclass(value=[Elm1(elm1="foo"), Elm2(elm2="bar")]))
        self.assertEqual(schema.dump(load), data_in)

    def test_union_list(self):
        @dataclass
        class Elm1:
            elm1: int

        @dataclass
        class Elm2:
            elm2: int

        @dataclass
        class TestDataClass:
            value: Union[List[Elm1], List[Elm2]]

        schema = TestDataClass.Schema()

        data_in = {"value": [{"elm1": 10}, {"elm1": 11}]}
        load = schema.load(data_in)
        self.assertEqual(load, TestDataClass(value=[Elm1(elm1=10), Elm1(elm1=11)]))
        self.assertEqual(schema.dump(load), data_in)

        data_in = {"value": [{"elm2": 10}, {"elm2": 11}]}
        load = schema.load(data_in)
        self.assertEqual(load, TestDataClass(value=[Elm2(elm2=10), Elm2(elm2=11)]))
        self.assertEqual(schema.dump(load), data_in)

        dictwrong_in = {"value": [{"elm1": 10}, {"elm2": 11}]}
        with self.assertRaises(marshmallow.exceptions.ValidationError):
            schema.load(dictwrong_in)

    def test_many_nested_union(self):
        @dataclass
        class Elm1:
            elm1: str

        @dataclass
        class Dclass:
            value: List[Union[List[Union[int, str, Elm1]], int]]

        schema = Dclass.Schema()
        data_in = {"value": [42, ["hello", 13, {"elm1": "foo"}]]}

        self.assertEqual(schema.dump(schema.load(data_in)), data_in)
        with self.assertRaises(marshmallow.exceptions.ValidationError):
            schema.load({"value": [42, ["hello", 13, {"elm2": "foo"}]]})

    def test_union_dict(self):
        @dataclass
        class Dclass:
            value: List[Union[Dict[int, Union[int, str]], Union[int, str]]]

        schema = Dclass.Schema()
        data_in = {"value": [42, {12: 13, 13: "hello"}, "foo"]}

        self.assertEqual(schema.dump(schema.load(data_in)), data_in)

        with self.assertRaises(marshmallow.exceptions.ValidationError):
            schema.load({"value": [(42,), {12: 13, 13: "hello"}, "foo"]})

    def test_union_list_dict(self):
        @dataclass
        class Elm:
            elm: int

        @dataclass
        class Dclass:
            value: Union[List[int], Dict[str, Elm]]

        schema = Dclass.Schema()

        data_in = {"value": {"a": {"elm": 10}, "b": {"elm": 10}}}
        self.assertEqual(schema.dump(schema.load(data_in)), data_in)

        data_in = {"value": [1, 2, 3, 4]}
        self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_union_noschema(self):
        @dataclass
        class Dclass:
            value: Union[int, str]

        schema = Dclass.Schema()
        data_in = {"value": [1.4, 4.2]}
        with self.assertRaises(marshmallow.exceptions.ValidationError):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_union_optional_object(self):
        @dataclass
        class Elm1:
            elm1: str

        @dataclass
        class Elm2:
            elm2: str

        @dataclass
        class Dclass:
            value: Optional[Union[Elm1, Elm2]]

        schema = Dclass.Schema()

        for data_in in [{"value": {"elm1": "hello"}}, {"value": {"elm2": "hello"}}]:
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

        for data_in in [{"value": None}, {}]:
            self.assertEqual(schema.dump(schema.load(data_in)), {"value": None})

    def test_required_optional_simple_union(self):
        @dataclass
        class Dclass:
            value: Optional[Union[int, str]] = field(metadata={"required": True})

        schema = Dclass.Schema()

        for value in None, 42, "strvar":
            self.assertEqual(schema.dump(Dclass(value=value)), {"value": value})
            self.assertEqual(schema.load({"value": value}), Dclass(value=value))

    def test_union_with_default(self):
        @dataclass
        class IntOrStrWithDefault:
            value: Union[int, str] = 42

        schema = IntOrStrWithDefault.Schema()
        self.assertEqual(schema.load({}), IntOrStrWithDefault(value=42))
        for value in 13, "strval":
            self.assertEqual(
                schema.load({"value": value}), IntOrStrWithDefault(value=value)
            )
        with self.assertRaises(marshmallow.exceptions.ValidationError):
            schema.load({"value": None})

    @unittest.skipIf(sys.version_info < (3, 10), "No PEP604 support in py<310")
    def test_pep604_union(self):
        @dataclass
        class PEP604IntOrStr:
            value: int | str

        schema = PEP604IntOrStr.Schema()
        data_in = {"value": "hello"}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, PEP604IntOrStr(value="hello"))
        self.assertEqual(schema.dump(loaded), data_in)

        data_in = {"value": 42}
        self.assertEqual(schema.dump(schema.load(data_in)), data_in)



import unittest
from typing import List, Optional

from marshmallow_dataclass import dataclass


@dataclass
class GlobalA:
    b: "GlobalB"


@dataclass
class GlobalB:
    pass


@dataclass
class GlobalSelfRecursion:
    related: "List[GlobalSelfRecursion]"


@dataclass
class GlobalRecursion:
    related: "List[GlobalRecursion]"


@dataclass
class GlobalCyclicA:
    b: "Optional[GlobalCyclicB]"


@dataclass
class GlobalCyclicB:
    a: "Optional[GlobalCyclicA]"


class TestForwardReferences(unittest.TestCase):
    def test_late_evaluated_types(self):
        @dataclass
        class MyData:
            value: int

        self.assertEqual(MyData(1), MyData.Schema().load(dict(value=1)))

    def test_forward_references_for_basic_types(self):
        @dataclass
        class Person:
            name: "str"
            age: "int"

        self.assertEqual(
            Person("Jon", 25), Person.Schema().load(dict(name="Jon", age=25))
        )

    def test_global_forward_references(self):
        self.assertEqual(GlobalA(GlobalB()), GlobalA.Schema().load(dict(b=dict())))

    def test_global_self_recursive_type(self):
        self.assertEqual(
            GlobalSelfRecursion([GlobalSelfRecursion([])]),
            GlobalSelfRecursion.Schema().load(dict(related=[dict(related=[])])),
        )

    def test_global_recursive_type(self):
        self.assertEqual(
            GlobalRecursion([GlobalRecursion([])]),
            GlobalRecursion.Schema().load(dict(related=[dict(related=[])])),
        )

    def test_global_circular_reference(self):
        self.assertEqual(
            GlobalCyclicA(GlobalCyclicB(GlobalCyclicA(None))),
            GlobalCyclicA.Schema().load(dict(b=dict(a=dict(b=None)))),
        )

    def test_local_self_recursive_type(self):
        @dataclass
        class LocalSelfRecursion:
            related: "List[LocalSelfRecursion]"

        self.assertEqual(
            LocalSelfRecursion([LocalSelfRecursion([])]),
            LocalSelfRecursion.Schema().load(dict(related=[dict(related=[])])),
        )

    def test_local_recursive_type(self):
        @dataclass
        class LocalRecursion:
            related: "List[LocalRecursion]"

        self.assertEqual(
            LocalRecursion([LocalRecursion([])]),
            LocalRecursion.Schema().load(dict(related=[dict(related=[])])),
        )

    def test_local_forward_references(self):
        @dataclass
        class LocalA:
            b: "LocalB"

        @dataclass
        class LocalB:
            pass

        self.assertEqual(LocalA(LocalB()), LocalA.Schema().load(dict(b=dict())))

    def test_name_collisions(self):
        """
        This is one example about why you should not make local schemas
        :return:
        """

        def make_another_a():
            @dataclass
            class A:
                d: int

            A.Schema()

        make_another_a()

        @dataclass
        class A:
            c: int

        A.Schema()

        @dataclass
        class B:
            a: "A"

        # with self.assertRaises(marshmallow.exceptions.ValidationError):
        B.Schema().load(dict(a=dict(c=1)))
        # marshmallow.exceptions.ValidationError:
        # {'a': {'d': ['Missing data for required field.'], 'c': ['Unknown field.']}}



import unittest

import marshmallow

import marshmallow_dataclass


# Regression test for https://github.com/lovasoa/marshmallow_dataclass/issues/75
class TestPostLoad(unittest.TestCase):
    @marshmallow_dataclass.dataclass
    class Named:
        first_name: str
        last_name: str

        @marshmallow.post_load
        def a(self, data, **_kwargs):
            data["first_name"] = data["first_name"].capitalize()
            return data

        @marshmallow.post_load
        def z(self, data, **_kwargs):
            data["last_name"] = data["last_name"].capitalize()
            return data

    def test_post_load_method_naming_does_not_affect_data(self):
        actual = self.Named.Schema().load(
            {"first_name": "matt", "last_name": "groening"}
        )
        expected = self.Named(first_name="Matt", last_name="Groening")
        self.assertEqual(actual, expected)

    def test_load_many(self):
        actual = self.Named.Schema().load(
            [
                {"first_name": "matt", "last_name": "groening"},
                {"first_name": "bart", "last_name": "simpson"},
            ],
            many=True,
        )
        expected = [
            self.Named(first_name="Matt", last_name="Groening"),
            self.Named(first_name="Bart", last_name="Simpson"),
        ]
        self.assertEqual(actual, expected)



import copy
from typing import List, Tuple, Any, Optional

import typeguard
from marshmallow import fields, Schema, ValidationError


class Union(fields.Field):
    """A union field, composed other `Field` classes or instances.
    This field serializes elements based on their type, with one of its child fields.

    Example: ::

        number_or_string = UnionField([
                    (float, fields.Float()),
                    (str, fields.Str())
                ])

    :param union_fields: A list of types and their associated field instance.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    def __init__(self, union_fields: List[Tuple[type, fields.Field]], **kwargs):
        super().__init__(**kwargs)
        self.union_fields = union_fields

    def _bind_to_schema(self, field_name: str, schema: Schema) -> None:
        super()._bind_to_schema(field_name, schema)
        new_union_fields = []
        for typ, field in self.union_fields:
            field = copy.deepcopy(field)
            field._bind_to_schema(field_name, self)
            new_union_fields.append((typ, field))

        self.union_fields = new_union_fields

    def _serialize(self, value: Any, attr: Optional[str], obj, **kwargs) -> Any:
        errors = []
        if value is None:
            return value
        for typ, field in self.union_fields:
            try:
                typeguard.check_type(
                    value=value, expected_type=typ, argname=attr or "anonymous"
                )
                return field._serialize(value, attr, obj, **kwargs)
            except TypeError as e:
                errors.append(e)
        raise TypeError(
            f"Unable to serialize value with any of the fields in the union: {errors}"
        )

    def _deserialize(self, value: Any, attr: Optional[str], data, **kwargs) -> Any:
        errors = []
        for typ, field in self.union_fields:
            try:
                result = field.deserialize(value, **kwargs)
                typeguard.check_type(
                    value=result, expected_type=typ, argname=attr or "anonymous"
                )
                return result
            except (TypeError, ValidationError) as e:
                errors.append(e)

        raise ValidationError(errors)



import doctest

import marshmallow_dataclass


def load_tests(_loader, tests, _ignore):
    # Load all the doctests defined in the module
    tests.addTests(doctest.DocTestSuite(marshmallow_dataclass))
    return tests



import itertools
import sys
import unittest
from typing import Optional, Sequence, Set, FrozenSet, Mapping

import marshmallow

from marshmallow_dataclass import dataclass


class TestSequenceField(unittest.TestCase):
    def test_simple(self):
        @dataclass
        class IntSequence:
            value: Sequence[int]

        schema = IntSequence.Schema()

        # can load a sequence of ints
        data_in = {"value": list(range(5))}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, IntSequence(value=tuple(range(5))))
        self.assertEqual(schema.dump(loaded), data_in)

        # can load a empty sequence
        data_in = {"value": []}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, IntSequence(value=()))
        self.assertEqual(schema.dump(loaded), data_in)

        # any sequence can be dumped (any object that can be iterated in order)
        for sequence in ((1, 2, 3), {1: "a", 2: "b", 3: "c"}, [1, 2, 3]):
            self.assertEqual(
                schema.dump(IntSequence(value=sequence)), {"value": [1, 2, 3]}
            )
        # does not allow None
        with self.assertRaises(marshmallow.exceptions.ValidationError):
            schema.load({"value": None})

        # should only accept int
        for data_not_int in ({"value": [None]}, {"value": ["a"]}):
            with self.assertRaises(marshmallow.exceptions.ValidationError):
                schema.load(data_not_int)

    def test_sequence_no_arg(self):
        @dataclass
        class AnySequence:
            value: Sequence

        schema = AnySequence.Schema()

        # can load a sequence of mixed kind
        data_in = {"value": [1, "2", 3.0, True, None, {"this", "is a", "set"}]}
        loaded = schema.load(data_in)
        self.assertEqual(
            loaded,
            AnySequence(value=(1, "2", 3.0, True, None, {"this", "is a", "set"})),
        )
        self.assertEqual(schema.dump(loaded), data_in)

    def test_optional_sequence(self):
        @dataclass
        class IntOptionalSequence:
            value: Optional[Sequence[int]]

        schema = IntOptionalSequence.Schema()
        for data_in in ({"value": list(range(5))}, {"value": []}, {"value": None}):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_sequence_of_optional(self):
        @dataclass
        class OptionalIntSequence:
            value: Sequence[Optional[int]]

        schema = OptionalIntSequence.Schema()
        for data_in in ({"value": list(range(5))}, {"value": [1, None, 2, None]}):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_nested_sequence(self):
        @dataclass
        class SequenceIntSequence:
            value: Sequence[Sequence[int]]

        schema = SequenceIntSequence.Schema()
        for data_in in (
            {"value": []},
            {"value": [[], []]},
            {"value": [[1, 2, 3], [4, 5, 6]]},
        ):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_sequence_of_dataclass(self):
        @dataclass
        class Elm:
            value: int

        @dataclass
        class SequenceIntSequence:
            value: Sequence[Elm]

        schema = SequenceIntSequence.Schema()
        for data_in in (
            {"value": []},
            {"value": [{"value": 1}]},
        ):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_sequence_of_optional_dataclass(self):
        @dataclass
        class Elm:
            value: int

        @dataclass
        class SequenceIntSequence:
            value: Sequence[Optional[Elm]]

        schema = SequenceIntSequence.Schema()
        for data_in in (
            {"value": []},
            {"value": [None]},
            {"value": [{"value": 1}]},
        ):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)


class TestSetField(unittest.TestCase):
    def assertEqualAsSet(self, a, b):
        self.assertEqual(set(a), set(b))

    def test_simple(self):
        @dataclass
        class IntSet:
            value: Set[int]

        schema = IntSet.Schema()
        data_in = {"value": list(range(5))}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, IntSet(value=set(range(5))))
        self.assertEqualAsSet(schema.dump(loaded)["value"], data_in["value"])

    def test_set_no_arg(self):
        @dataclass
        class AnySet:
            value: Set

        schema = AnySet.Schema()

        # can load a set of mixed kind
        data_in = {"value": [1, "2", 3.0, (1, 2, 3)]}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, AnySet(value={1, "2", 3.0, (1, 2, 3)}))
        self.assertEqualAsSet(schema.dump(loaded)["value"], data_in["value"])

    def test_optional_set(self):
        @dataclass
        class IntOptionalSet:
            value: Optional[Set[int]]

        schema = IntOptionalSet.Schema()

        # None can be loaded and dumped
        data_in = {"value": None}
        self.assertEqual(schema.dump(schema.load(data_in)), data_in)

        # and regular values too
        data_in = {"value": list(range(5))}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, IntOptionalSet(value=set(range(5))))
        self.assertEqualAsSet(schema.dump(loaded)["value"], data_in["value"])

    def test_set_of_optional(self):
        @dataclass
        class OptionalIntSet:
            value: Set[Optional[int]]

        schema = OptionalIntSet.Schema()
        for data_in in ({"value": set(range(5))}, {"value": {1, None, 2}}):
            self.assertEqual(
                set(schema.dump(schema.load(data_in))["value"]), data_in["value"]
            )

    def test_set_only_work_in_hashable_types(self):
        @dataclass
        class SetIntSet:
            value: Set[Set[int]]

        schema = SetIntSet.Schema()

        with self.assertRaises(TypeError) as err_info:
            schema.load({"value": {set()}})
        self.assertEqual(str(err_info.exception), "unhashable type: 'set'")

        @dataclass()
        class Elm:
            value: int

        @dataclass
        class SetOfDataclass:
            value: Set[Elm]

        schema = SetOfDataclass.Schema()

        with self.assertRaises(TypeError) as err_info:
            schema.load({"value": {{"value": {set()}}}})
        self.assertEqual(str(err_info.exception), "unhashable type: 'set'")

    def test_set_of_frozen_dataclass(self):
        @dataclass(frozen=True)
        class Elm:
            value: str

        @dataclass
        class SetIntSet:
            value: Set[Elm]

        schema = SetIntSet.Schema()
        for data_in in (
            {"value": []},
            {"value": [{"value": "john"}, {"value": "doe"}, {"value": "alex"}]},
        ):
            # All the items have to be the same, but the order is not guaranteed
            self.assertIn(
                schema.dump(schema.load(data_in))["value"],
                map(list, itertools.permutations(data_in["value"])),
            )

    def test_set_of_optional_dataclass(self):
        @dataclass(frozen=True)
        class Elm:
            value: int

        @dataclass
        class SetIntSet:
            value: Set[Optional[Elm]]

        schema = SetIntSet.Schema()
        for data_in in (
            {"value": []},
            {"value": [None]},
            {"value": [{"value": 1}]},
        ):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)

    def test_frozen_set(self):
        @dataclass
        class IntFrozenSet:
            value: FrozenSet[int]

        schema = IntFrozenSet.Schema()
        data_in = {"value": list(range(5))}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, IntFrozenSet(value=frozenset(range(5))))
        self.assertEqual(schema.dump(loaded), data_in)

    def test_nested_frozenset(self):
        """frozen sets are hashable so we can nest them"""

        @dataclass
        class SetIntSet:
            value: FrozenSet[FrozenSet[int]]

        schema = SetIntSet.Schema()
        data_in = {
            "value": [
                [1, 2, 3],
                [3, 2, 1],
                [1, 3, 2],
                [2, 3, 1],
                [3, 1, 2],
                [2, 1, 3],
                [123],
            ]
        }
        loaded = schema.load(data_in)
        self.assertEqual(
            loaded,
            SetIntSet(value=frozenset([frozenset([1, 2, 3]), frozenset([123])])),
        )
        self.assertEqualAsSet(schema.dump(loaded), data_in)


class TestMappingField(unittest.TestCase):
    def test_simple(self):
        @dataclass
        class StrIntMapping:
            value: Mapping[str, int]

        schema = StrIntMapping.Schema()
        value = {key: value for key, value in zip("abcdefghi", range(5))}
        data_in = {"value": value}
        loaded = schema.load(data_in)
        self.assertEqual(loaded, StrIntMapping(value=value))
        self.assertEqual(schema.dump(loaded), data_in)

    def test_mapping_no_arg(self):
        @dataclass
        class AnyMapping:
            value: Mapping

        schema = AnyMapping.Schema()

        # can load a sequence of mixed kind
        data_in = {
            "value": {
                1: "a number key a str value",
                "a str key a number value": 2,
                None: "this is still valid",
                "even this": None,
            }
        }
        loaded = schema.load(data_in)
        self.assertEqual(
            loaded,
            AnyMapping(
                value={
                    1: "a number key a str value",
                    "a str key a number value": 2,
                    None: "this is still valid",
                    "even this": None,
                }
            ),
        )
        self.assertEqual(schema.dump(loaded), data_in)

    def test_mapping_of_frozen_dataclass(self):
        @dataclass(frozen=True)
        class Elm:
            name: str

        @dataclass
        class DataclassMapping:
            value: Mapping[int, Elm]

        schema = DataclassMapping.Schema()
        for data_in in (
            {"value": {1: {"name": "John"}}},
            {"value": {1: {"name": "Doe"}}},
        ):
            self.assertEqual(schema.dump(schema.load(data_in)), data_in)


class TestBuiltinGenerics(unittest.TestCase):
    @unittest.skipIf(
        sys.version_info < (3, 9),
        "builtin generics are only available in python 3.9 upwards",
    )
    def test_builtin(self):
        annotations = [list[int], dict[str, int], set[str]]
        samples = [[1, 2, 3], {"foo": 1, "bar": 2}, {"f", "o", "b", "a", "r"}]

        for annotation, sample in zip(annotations, samples):
            with self.subTest(f"Testing {annotation}"):

                @dataclass
                class Dummy:
                    value: annotation

                schema = Dummy.Schema()

                # check if a round trip keeps the data untouched
                loaded = schema.load(schema.dump(Dummy(value=sample)))
                self.assertEqual(loaded, Dummy(value=sample))



import unittest
from dataclasses import field
from typing import Optional

import marshmallow

from marshmallow_dataclass import dataclass


class TestOptionalField(unittest.TestCase):
    def test_optional_field(self):
        @dataclass
        class OptionalValue:
            value: Optional[str] = "value"

        schema = OptionalValue.Schema()

        self.assertEqual(schema.load({"value": None}), OptionalValue(value=None))
        self.assertEqual(schema.load({"value": "hello"}), OptionalValue(value="hello"))
        self.assertEqual(schema.load({}), OptionalValue())

    def test_optional_field_not_none(self):
        @dataclass
        class OptionalValueNotNone:
            value: Optional[str] = field(
                default="value", metadata={"allow_none": False}
            )

        schema = OptionalValueNotNone.Schema()

        self.assertEqual(schema.load({}), OptionalValueNotNone())
        self.assertEqual(
            schema.load({"value": "hello"}), OptionalValueNotNone(value="hello")
        )
        with self.assertRaises(marshmallow.exceptions.ValidationError) as exc_cm:
            schema.load({"value": None})
        self.assertEqual(
            exc_cm.exception.messages, {"value": ["Field may not be null."]}
        )

    def test_required_optional_field(self):
        @dataclass
        class RequiredOptionalValue:
            value: Optional[str] = field(default="default", metadata={"required": True})

        schema = RequiredOptionalValue.Schema()

        self.assertEqual(schema.load({"value": None}), RequiredOptionalValue(None))
        self.assertEqual(
            schema.load({"value": "hello"}), RequiredOptionalValue(value="hello")
        )
        with self.assertRaises(marshmallow.exceptions.ValidationError) as exc_cm:
            schema.load({})
        self.assertEqual(
            exc_cm.exception.messages, {"value": ["Missing data for required field."]}
        )



import dataclasses
import unittest

import marshmallow.validate

from marshmallow_dataclass import class_schema


class TestAttributesCopy(unittest.TestCase):
    def test_meta_class_copied(self):
        @dataclasses.dataclass
        class Anything:
            class Meta:
                pass

        schema = class_schema(Anything)
        self.assertEqual(schema.Meta, Anything.Meta)

    def test_validates_schema_copied(self):
        @dataclasses.dataclass
        class Anything:
            @marshmallow.validates_schema
            def validates_schema(self, *args, **kwargs):
                pass

        schema = class_schema(Anything)
        self.assertIn("validates_schema", dir(schema))

    def test_custom_method_not_copied(self):
        @dataclasses.dataclass
        class Anything:
            def custom_method(self):
                pass

        schema = class_schema(Anything)
        self.assertNotIn("custom_method", dir(schema))

    def test_custom_property_not_copied(self):
        @dataclasses.dataclass
        class Anything:
            @property
            def custom_property(self):
                return 42

        schema = class_schema(Anything)
        self.assertNotIn("custom_property", dir(schema))


if __name__ == "__main__":
    unittest.main()



import inspect
import sys
import typing
import unittest
from enum import Enum
from typing import Dict, Optional, Union, Any, List, Tuple

try:
    from typing import Final, Literal  # type: ignore[attr-defined]
except ImportError:
    from typing_extensions import Final, Literal  # type: ignore[misc]

from marshmallow import fields, Schema, validate

from marshmallow_dataclass import (
    field_for_schema,
    dataclass,
    union_field,
    collection_field,
)


class TestFieldForSchema(unittest.TestCase):
    def assertFieldsEqual(self, a: fields.Field, b: fields.Field):
        self.assertEqual(a.__class__, b.__class__, "field class")

        def attrs(x):
            return {
                k: f"{v!r} ({v.__mro__!r})" if inspect.isclass(v) else repr(v)
                for k, v in x.__dict__.items()
                if not k.startswith("_")
            }

        self.assertEqual(attrs(a), attrs(b))

    def test_int(self):
        self.assertFieldsEqual(
            field_for_schema(int, default=9, metadata=dict(required=False)),
            fields.Integer(dump_default=9, load_default=9, required=False),
        )

    def test_any(self):
        self.assertFieldsEqual(
            field_for_schema(Any), fields.Raw(required=True, allow_none=True)
        )

    def test_dict_from_typing(self):
        self.assertFieldsEqual(
            field_for_schema(Dict[str, float]),
            fields.Dict(
                keys=fields.String(required=True),
                values=fields.Float(required=True),
                required=True,
            ),
        )

    def test_dict_from_typing_wo_args(self):
        self.assertFieldsEqual(
            field_for_schema(Dict),
            fields.Dict(
                keys=fields.Raw(required=True, allow_none=True),
                values=fields.Raw(required=True, allow_none=True),
                required=True,
            ),
        )

    def test_builtin_dict(self):
        self.assertFieldsEqual(
            field_for_schema(dict),
            fields.Dict(
                keys=fields.Raw(required=True, allow_none=True),
                values=fields.Raw(required=True, allow_none=True),
                required=True,
            ),
        )

    def test_list_from_typing(self):
        self.assertFieldsEqual(
            field_for_schema(List[int]),
            fields.List(fields.Integer(required=True), required=True),
        )

    def test_list_from_typing_wo_args(self):
        self.assertFieldsEqual(
            field_for_schema(List),
            fields.List(
                fields.Raw(required=True, allow_none=True),
                required=True,
            ),
        )

    def test_builtin_list(self):
        self.assertFieldsEqual(
            field_for_schema(list, metadata=dict(required=False)),
            fields.List(fields.Raw(required=True, allow_none=True), required=False),
        )

    def test_explicit_field(self):
        explicit_field = fields.Url(required=True)
        self.assertFieldsEqual(
            field_for_schema(str, metadata={"marshmallow_field": explicit_field}),
            explicit_field,
        )

    def test_str(self):
        self.assertFieldsEqual(field_for_schema(str), fields.String(required=True))

    def test_optional_str(self):
        self.assertFieldsEqual(
            field_for_schema(Optional[str]),
            fields.String(
                allow_none=True, required=False, dump_default=None, load_default=None
            ),
        )

    def test_enum(self):
        import marshmallow_enum

        class Color(Enum):
            RED: 1
            GREEN: 2
            BLUE: 3

        self.assertFieldsEqual(
            field_for_schema(Color),
            marshmallow_enum.EnumField(enum=Color, required=True),
        )

    def test_literal(self):
        self.assertFieldsEqual(
            field_for_schema(Literal["a"]),
            fields.Raw(required=True, validate=validate.Equal("a")),
        )

    def test_literal_multiple_types(self):
        self.assertFieldsEqual(
            field_for_schema(Literal["a", 1, 1.23, True]),
            fields.Raw(required=True, validate=validate.OneOf(("a", 1, 1.23, True))),
        )

    def test_final(self):
        self.assertFieldsEqual(
            field_for_schema(Final[str]), fields.String(required=True)
        )

    def test_final_without_type(self):
        self.assertFieldsEqual(
            field_for_schema(Final), fields.Raw(required=True, allow_none=True)
        )

    def test_union(self):
        self.assertFieldsEqual(
            field_for_schema(Union[int, str]),
            union_field.Union(
                [
                    (int, fields.Integer(required=True)),
                    (str, fields.String(required=True)),
                ],
                required=True,
            ),
        )

    def test_union_multiple_types_with_none(self):
        self.assertFieldsEqual(
            field_for_schema(Union[int, str, None]),
            union_field.Union(
                [
                    (int, fields.Integer(required=True)),
                    (str, fields.String(required=True)),
                ],
                required=False,
                dump_default=None,
                load_default=None,
            ),
        )

    def test_optional_multiple_types(self):
        self.assertFieldsEqual(
            field_for_schema(Optional[Union[int, str]]),
            union_field.Union(
                [
                    (int, fields.Integer(required=True)),
                    (str, fields.String(required=True)),
                ],
                required=False,
                dump_default=None,
                load_default=None,
            ),
        )

    def test_newtype(self):
        self.assertFieldsEqual(
            field_for_schema(typing.NewType("UserId", int), default=0),
            fields.Integer(
                required=False,
                dump_default=0,
                load_default=0,
                metadata={"description": "UserId"},
            ),
        )

    def test_marshmallow_dataclass(self):
        class NewSchema(Schema):
            pass

        @dataclass(base_schema=NewSchema)
        class NewDataclass:
            pass

        self.assertFieldsEqual(
            field_for_schema(NewDataclass, metadata=dict(required=False)),
            fields.Nested(NewDataclass.Schema),
        )

    def test_override_container_type_with_type_mapping(self):
        type_mapping = [
            (List, fields.List, List[int]),
            (Dict, fields.Dict, Dict[str, int]),
            (Tuple, fields.Tuple, Tuple[int, str, bytes]),
        ]
        for base_type, marshmallow_field, schema in type_mapping:

            class MyType(marshmallow_field):
                ...

            self.assertIsInstance(field_for_schema(schema), marshmallow_field)

            class BaseSchema(Schema):
                TYPE_MAPPING = {base_type: MyType}

            self.assertIsInstance(
                field_for_schema(schema, base_schema=BaseSchema), MyType
            )

    def test_mapping(self):
        self.assertFieldsEqual(
            field_for_schema(typing.Mapping),
            fields.Dict(
                keys=fields.Raw(required=True, allow_none=True),
                values=fields.Raw(required=True, allow_none=True),
                required=True,
            ),
        )

    def test_sequence(self):
        self.maxDiff = 2000
        self.assertFieldsEqual(
            field_for_schema(typing.Sequence[int]),
            collection_field.Sequence(fields.Integer(required=True), required=True),
        )

    def test_sequence_wo_args(self):
        self.assertFieldsEqual(
            field_for_schema(typing.Sequence),
            collection_field.Sequence(
                cls_or_instance=fields.Raw(required=True, allow_none=True),
                required=True,
            ),
        )

    def test_homogeneous_tuple_from_typing(self):
        self.assertFieldsEqual(
            field_for_schema(Tuple[str, ...]),
            collection_field.Sequence(fields.String(required=True), required=True),
        )

    @unittest.skipIf(sys.version_info < (3, 9), "PEP 585 unsupported")
    def test_homogeneous_tuple(self):
        self.assertFieldsEqual(
            field_for_schema(tuple[float, ...]),
            collection_field.Sequence(fields.Float(required=True), required=True),
        )

    def test_set_from_typing(self):
        self.assertFieldsEqual(
            field_for_schema(typing.Set[str]),
            collection_field.Set(
                fields.String(required=True),
                frozen=False,
                required=True,
            ),
        )

    def test_set_from_typing_wo_args(self):
        self.assertFieldsEqual(
            field_for_schema(typing.Set),
            collection_field.Set(
                cls_or_instance=fields.Raw(required=True, allow_none=True),
                frozen=False,
                required=True,
            ),
        )

    def test_builtin_set(self):
        self.assertFieldsEqual(
            field_for_schema(set),
            collection_field.Set(
                cls_or_instance=fields.Raw(required=True, allow_none=True),
                frozen=False,
                required=True,
            ),
        )

    def test_frozenset_from_typing(self):
        self.assertFieldsEqual(
            field_for_schema(typing.FrozenSet[int]),
            collection_field.Set(
                fields.Integer(required=True),
                frozen=True,
                required=True,
            ),
        )

    def test_frozenset_from_typing_wo_args(self):
        self.assertFieldsEqual(
            field_for_schema(typing.FrozenSet),
            collection_field.Set(
                cls_or_instance=fields.Raw(required=True, allow_none=True),
                frozen=True,
                required=True,
            ),
        )

    def test_builtin_frozenset(self):
        self.assertFieldsEqual(
            field_for_schema(frozenset),
            collection_field.Set(
                cls_or_instance=fields.Raw(required=True, allow_none=True),
                frozen=True,
                required=True,
            ),
        )


if __name__ == "__main__":
    unittest.main()



import inspect
import typing
import unittest
from typing import Any, cast, TYPE_CHECKING
from uuid import UUID

try:
    from typing import Final, Literal  # type: ignore[attr-defined]
except ImportError:
    from typing_extensions import Final, Literal  # type: ignore[misc]

import dataclasses
from marshmallow import Schema, ValidationError
from marshmallow.fields import Field, UUID as UUIDField, List as ListField, Integer
from marshmallow.validate import Validator

from marshmallow_dataclass import class_schema, NewType


class TestClassSchema(unittest.TestCase):
    def test_simple_unique_schemas(self):
        @dataclasses.dataclass
        class Simple:
            one: str = dataclasses.field()
            two: str = dataclasses.field()

        @dataclasses.dataclass
        class ComplexNested:
            three: int = dataclasses.field()
            four: Simple = dataclasses.field()

        self.assertIs(class_schema(ComplexNested), class_schema(ComplexNested))
        self.assertIs(class_schema(Simple), class_schema(Simple))
        self.assertIs(
            class_schema(Simple),
            class_schema(ComplexNested)._declared_fields["four"].nested,
        )

        complex_set = {
            class_schema(ComplexNested),
            class_schema(ComplexNested, base_schema=None),
            class_schema(ComplexNested, clazz_frame=None),
            class_schema(ComplexNested, None),
            class_schema(ComplexNested, None, None),
        }
        simple_set = {
            class_schema(Simple),
            class_schema(Simple, base_schema=None),
            class_schema(Simple, clazz_frame=None),
            class_schema(Simple, None),
            class_schema(Simple, None, None),
        }
        self.assertEqual(len(complex_set), 1)
        self.assertEqual(len(simple_set), 1)

    def test_nested_schema_with_passed_frame(self):
        @dataclasses.dataclass
        class Simple:
            one: str = dataclasses.field()
            two: str = dataclasses.field()

        @dataclasses.dataclass
        class ComplexNested:
            three: int = dataclasses.field()
            four: Simple = dataclasses.field()

        frame = inspect.stack()[0][0]

        self.assertIs(
            class_schema(ComplexNested, clazz_frame=frame),
            class_schema(ComplexNested, clazz_frame=frame),
        )
        self.assertIs(
            class_schema(Simple, clazz_frame=frame),
            class_schema(Simple, clazz_frame=frame),
        )
        self.assertIs(
            class_schema(Simple, clazz_frame=frame),
            class_schema(ComplexNested, clazz_frame=frame)
            ._declared_fields["four"]
            .nested,
        )

        complex_set = {
            class_schema(ComplexNested, clazz_frame=frame),
            class_schema(ComplexNested, base_schema=None, clazz_frame=frame),
            class_schema(ComplexNested, None, clazz_frame=frame),
            class_schema(ComplexNested, None, frame),
        }
        simple_set = {
            class_schema(Simple, clazz_frame=frame),
            class_schema(Simple, base_schema=None, clazz_frame=frame),
            class_schema(Simple, None, clazz_frame=frame),
            class_schema(Simple, clazz_frame=frame),
            class_schema(Simple, None, frame),
        }
        self.assertEqual(len(complex_set), 1)
        self.assertEqual(len(simple_set), 1)

    def test_use_type_mapping_from_base_schema(self):
        class CustomType:
            pass

        class CustomField(Field):
            pass

        class CustomListField(ListField):
            pass

        class BaseSchema(Schema):
            TYPE_MAPPING = {CustomType: CustomField, typing.List: CustomListField}

        @dataclasses.dataclass
        class WithCustomField:
            custom: CustomType
            custom_list: typing.List[float]
            uuid: UUID
            n: int

        schema = class_schema(WithCustomField, base_schema=BaseSchema)()
        self.assertIsInstance(schema.fields["custom"], CustomField)
        self.assertIsInstance(schema.fields["custom_list"], CustomListField)
        self.assertIsInstance(schema.fields["uuid"], UUIDField)
        self.assertIsInstance(schema.fields["n"], Integer)

    def test_filtering_list_schema(self):
        class FilteringListField(ListField):
            def __init__(
                self,
                cls_or_instance: typing.Union[Field, type],
                min: typing.Any,
                max: typing.Any,
                **kwargs,
            ):
                super().__init__(cls_or_instance, **kwargs)
                self.min = min
                self.max = max

            def _deserialize(
                self, value, attr, data, **kwargs
            ) -> typing.List[typing.Any]:
                loaded = super()._deserialize(value, attr, data, **kwargs)
                return [x for x in loaded if self.min <= x <= self.max]

        class BaseSchema(Schema):
            TYPE_MAPPING = {typing.List: FilteringListField}

        @dataclasses.dataclass
        class WithCustomField:
            constrained_floats: typing.List[float] = dataclasses.field(
                metadata={"max": 2.5, "min": 1}
            )
            constrained_strings: typing.List[str] = dataclasses.field(
                metadata={"max": "x", "min": "a"}
            )

        schema = class_schema(WithCustomField, base_schema=BaseSchema)()
        actual = schema.load(
            {
                "constrained_floats": [0, 1, 2, 3],
                "constrained_strings": ["z", "a", "b", "c", ""],
            }
        )
        self.assertEqual(
            actual,
            WithCustomField(
                constrained_floats=[1.0, 2.0], constrained_strings=["a", "b", "c"]
            ),
        )

    def test_any_none(self):
        # See: https://github.com/lovasoa/marshmallow_dataclass/issues/80
        @dataclasses.dataclass
        class A:
            data: Any

        schema = class_schema(A)()
        self.assertEqual(A(data=None), schema.load({"data": None}))
        self.assertEqual(schema.dump(A(data=None)), {"data": None})

    def test_any_none_disallowed(self):
        @dataclasses.dataclass
        class A:
            data: Any = dataclasses.field(metadata={"allow_none": False})

        schema = class_schema(A)()
        self.assertRaises(ValidationError, lambda: schema.load({"data": None}))

    def test_literal(self):
        @dataclasses.dataclass
        class A:
            data: Literal["a"]

        schema = class_schema(A)()
        self.assertEqual(A(data="a"), schema.load({"data": "a"}))
        self.assertEqual(schema.dump(A(data="a")), {"data": "a"})
        for data in ["b", 2, 2.34, False]:
            with self.assertRaises(ValidationError):
                schema.load({"data": data})

    def test_literal_multiple_types(self):
        @dataclasses.dataclass
        class A:
            data: Literal["a", 1, 1.23, True]

        schema = class_schema(A)()
        for data in ["a", 1, 1.23, True]:
            self.assertEqual(A(data=data), schema.load({"data": data}))
            self.assertEqual(schema.dump(A(data=data)), {"data": data})
        for data in ["b", 2, 2.34, False]:
            with self.assertRaises(ValidationError):
                schema.load({"data": data})

    def test_final(self):
        @dataclasses.dataclass
        class A:
            # Mypy currently considers read-only dataclass attributes without a
            # default value an error.
            # See: https://github.com/python/mypy/issues/10688.
            data: Final[str]  # type: ignore[misc]

        schema = class_schema(A)()
        self.assertEqual(A(data="a"), schema.load({"data": "a"}))
        self.assertEqual(schema.dump(A(data="a")), {"data": "a"})
        for data in [2, 2.34, False]:
            with self.assertRaises(ValidationError):
                schema.load({"data": data})

    def test_final_infers_type_from_default(self):
        # @dataclasses.dataclass(frozen=True)
        class A:
            data: Final = "a"

        # @dataclasses.dataclass
        class B:
            data: Final = A()

        # NOTE: This workaround is needed to avoid a Mypy crash.
        # See: https://github.com/python/mypy/issues/10090#issuecomment-865971891
        if not TYPE_CHECKING:
            frozen_dataclass = dataclasses.dataclass(frozen=True)
            A = frozen_dataclass(A)
            B = dataclasses.dataclass(B)

        with self.assertWarns(Warning):
            schema_a = class_schema(A)()
        self.assertEqual(A(data="a"), schema_a.load({}))
        self.assertEqual(A(data="a"), schema_a.load({"data": "a"}))
        self.assertEqual(A(data="b"), schema_a.load({"data": "b"}))
        self.assertEqual(schema_a.dump(A()), {"data": "a"})
        self.assertEqual(schema_a.dump(A(data="a")), {"data": "a"})
        self.assertEqual(schema_a.dump(A(data="b")), {"data": "b"})
        for data in [2, 2.34, False]:
            with self.assertRaises(ValidationError):
                schema_a.load({"data": data})

        with self.assertWarns(Warning):
            schema_b = class_schema(B)()
        self.assertEqual(B(data=A()), schema_b.load({}))
        self.assertEqual(B(data=A()), schema_b.load({"data": {}}))
        self.assertEqual(B(data=A()), schema_b.load({"data": {"data": "a"}}))
        self.assertEqual(B(data=A(data="b")), schema_b.load({"data": {"data": "b"}}))
        self.assertEqual(schema_b.dump(B()), {"data": {"data": "a"}})
        self.assertEqual(schema_b.dump(B(data=A())), {"data": {"data": "a"}})
        self.assertEqual(schema_b.dump(B(data=A(data="a"))), {"data": {"data": "a"}})
        self.assertEqual(schema_b.dump(B(data=A(data="b"))), {"data": {"data": "b"}})
        for data in [2, 2.34, False]:
            with self.assertRaises(ValidationError):
                schema_b.load({"data": data})

    def test_final_infers_type_any_from_field_default_factory(self):
        # @dataclasses.dataclass
        class A:
            data: Final = dataclasses.field(default_factory=lambda: [])

        # NOTE: This workaround is needed to avoid a Mypy crash.
        # See: https://github.com/python/mypy/issues/10090#issuecomment-866686096
        if not TYPE_CHECKING:
            A = dataclasses.dataclass(A)

        with self.assertWarns(Warning):
            schema = class_schema(A)()

        self.assertEqual(A(data=[]), schema.load({}))
        self.assertEqual(A(data=[]), schema.load({"data": []}))
        self.assertEqual(A(data=["a"]), schema.load({"data": ["a"]}))
        self.assertEqual(schema.dump(A()), {"data": []})
        self.assertEqual(schema.dump(A(data=[])), {"data": []})
        self.assertEqual(schema.dump(A(data=["a"])), {"data": ["a"]})

        # The following test cases pass because the field type cannot be
        # inferred from a factory.
        self.assertEqual(A(data=cast(Any, True)), schema.load({"data": True}))
        self.assertEqual(A(data=cast(Any, "a")), schema.load({"data": "a"}))

    def test_validator_stacking(self):
        # See: https://github.com/lovasoa/marshmallow_dataclass/issues/91
        class SimpleValidator(Validator):
            # Marshmallow checks for valid validators at construction time only using `callable`
            def __call__(self):
                pass

        validator_a = SimpleValidator()
        validator_b = SimpleValidator()
        validator_c = SimpleValidator()
        validator_d = SimpleValidator()

        CustomTypeOneValidator = NewType(
            "CustomTypeOneValidator", str, validate=validator_a
        )
        CustomTypeNoneValidator = NewType("CustomTypeNoneValidator", str, validate=None)
        CustomTypeMultiValidator = NewType(
            "CustomTypeNoneValidator", str, validate=[validator_a, validator_b]
        )

        @dataclasses.dataclass
        class A:
            data: CustomTypeNoneValidator = dataclasses.field()

        schema_a = class_schema(A)()
        self.assertListEqual(schema_a.fields["data"].validators, [])

        @dataclasses.dataclass
        class B:
            data: CustomTypeNoneValidator = dataclasses.field(
                metadata={"validate": validator_a}
            )

        schema_b = class_schema(B)()
        self.assertListEqual(schema_b.fields["data"].validators, [validator_a])

        @dataclasses.dataclass
        class C:
            data: CustomTypeNoneValidator = dataclasses.field(
                metadata={"validate": [validator_a, validator_b]}
            )

        schema_c = class_schema(C)()
        self.assertListEqual(
            schema_c.fields["data"].validators, [validator_a, validator_b]
        )

        @dataclasses.dataclass
        class D:
            data: CustomTypeOneValidator = dataclasses.field()

        schema_d = class_schema(D)()
        self.assertListEqual(schema_d.fields["data"].validators, [validator_a])

        @dataclasses.dataclass
        class E:
            data: CustomTypeOneValidator = dataclasses.field(
                metadata={"validate": validator_b}
            )

        schema_e = class_schema(E)()
        self.assertListEqual(
            schema_e.fields["data"].validators, [validator_a, validator_b]
        )

        @dataclasses.dataclass
        class F:
            data: CustomTypeOneValidator = dataclasses.field(
                metadata={"validate": [validator_b, validator_c]}
            )

        schema_f = class_schema(F)()
        self.assertListEqual(
            schema_f.fields["data"].validators, [validator_a, validator_b, validator_c]
        )

        @dataclasses.dataclass
        class G:
            data: CustomTypeMultiValidator = dataclasses.field()

        schema_g = class_schema(G)()
        self.assertListEqual(
            schema_g.fields["data"].validators, [validator_a, validator_b]
        )

        @dataclasses.dataclass
        class H:
            data: CustomTypeMultiValidator = dataclasses.field(
                metadata={"validate": validator_c}
            )

        schema_h = class_schema(H)()
        self.assertListEqual(
            schema_h.fields["data"].validators, [validator_a, validator_b, validator_c]
        )

        @dataclasses.dataclass
        class J:
            data: CustomTypeMultiValidator = dataclasses.field(
                metadata={"validate": [validator_c, validator_d]}
            )

        schema_j = class_schema(J)()
        self.assertListEqual(
            schema_j.fields["data"].validators,
            [validator_a, validator_b, validator_c, validator_d],
        )

    def test_recursive_reference(self):
        @dataclasses.dataclass
        class Tree:
            children: typing.List["Tree"]  # noqa: F821

        schema = class_schema(Tree)()

        self.assertEqual(
            schema.load({"children": [{"children": []}]}),
            Tree(children=[Tree(children=[])]),
        )

    def test_cyclic_reference(self):
        @dataclasses.dataclass
        class First:
            second: typing.Optional["Second"]  # noqa: F821

        @dataclasses.dataclass
        class Second:
            first: typing.Optional["First"]

        first_schema = class_schema(First)()
        second_schema = class_schema(Second)()

        self.assertEqual(
            first_schema.load({"second": {"first": None}}),
            First(second=Second(first=None)),
        )
        self.assertEqual(
            second_schema.dump(Second(first=First(second=Second(first=None)))),
            {"first": {"second": {"first": None}}},
        )


if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages

VERSION = "8.5.10"

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Operating System :: OS Independent",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries",
]

EXTRAS_REQUIRE = {
    "enum": ["marshmallow-enum"],
    "union": ["typeguard"],
    "lint": ["pre-commit~=2.17"],
    ':python_version == "3.6"': ["dataclasses", "types-dataclasses<0.6.4"],
    "docs": ["sphinx"],
    "tests": [
        "pytest>=5.4",
        # re: pypy: typed-ast (a dependency of mypy) fails to install on pypy
        # https://github.com/python/typed_ast/issues/111
        "pytest-mypy-plugins>=1.2.0; implementation_name != 'pypy'",
        # `Literal` was introduced in:
        # - Python 3.8 (https://www.python.org/dev/peps/pep-0586)
        # - typing-extensions 3.7.2 (https://github.com/python/typing/pull/591)
        "typing-extensions>=3.7.2; python_version < '3.8'",
    ],
}
EXTRAS_REQUIRE["dev"] = (
    EXTRAS_REQUIRE["enum"]
    + EXTRAS_REQUIRE["union"]
    + EXTRAS_REQUIRE["lint"]
    + EXTRAS_REQUIRE["docs"]
    + EXTRAS_REQUIRE["tests"]
)

setup(
    name="marshmallow_dataclass",
    version=VERSION,
    description="Python library to convert dataclasses into marshmallow schemas.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(
        include=["marshmallow_dataclass", "marshmallow_dataclass.*"]
    ),
    author="Ophir LOJKINE",
    author_email="pere.jobs@gmail.com",
    url="https://github.com/lovasoa/marshmallow_dataclass",
    keywords=["marshmallow", "dataclass", "serialization"],
    classifiers=CLASSIFIERS,
    license="MIT",
    python_requires=">=3.6",
    install_requires=["marshmallow>=3.13.0,<4.0", "typing-inspect>=0.7.1"],
    extras_require=EXTRAS_REQUIRE,
    package_data={"marshmallow_dataclass": ["py.typed"]},
)



import typing as T
from dataclasses import field

import marshmallow as mm
import marshmallow_dataclass as mmdc
import collections.abc
import unittest


# Test from https://github.com/lovasoa/marshmallow_dataclass/issues/125
class TestPostDump(unittest.TestCase):
    def setUp(self) -> None:
        class BaseSchema(mm.Schema):
            SKIP_VALUES = {None}

            @mm.post_dump
            def remove_skip_values(self, data, many):
                return {
                    key: value
                    for key, value in data.items()
                    if not isinstance(value, collections.abc.Hashable)
                    or value not in self.SKIP_VALUES
                }

        @mmdc.add_schema(base_schema=BaseSchema)
        @mmdc.dataclass
        class Item:
            key: str
            val: str
            Schema: T.ClassVar[T.Type[mm.Schema]] = mm.Schema

        @mmdc.add_schema(base_schema=BaseSchema)
        @mmdc.dataclass
        class Items:
            other: T.Optional[str]
            items: T.List[Item] = field(default_factory=list)
            Schema: T.ClassVar[T.Type[mm.Schema]] = mm.Schema

        self.BaseSchema = BaseSchema
        self.Item = Item
        self.Items = Items
        self.data = {"items": [{"key": "any key", "val": "any value"}]}

    def test_class_schema(self):
        items_schema = mmdc.class_schema(self.Items, base_schema=self.BaseSchema)()
        items = items_schema.load(self.data)
        self.assertEqual(
            {"items": [{"key": "any key", "val": "any value"}]},
            items_schema.dump(items),
        )

    def test_inside_schema(self):
        items = self.Items.Schema().load(self.data)
        self.assertEqual(
            {"items": [{"key": "any key", "val": "any value"}]},
            self.Items.Schema().dump(items),
        )
